@@ -1,4 +1,4 @@
-/* $Id: miniupnpd.c,v 1.148 2012/03/05 20:36:16 nanard Exp $ */
+/* $Id: miniupnpd.c,v 1.161 2012/05/21 15:50:03 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2012 Thomas Bernard
@@ -49,6 +49,7 @@
 #include <sys/un.h>
 #endif
 
+#include "macros.h"
 #include "upnpglobalvars.h"
 #include "upnphttp.h"
 #include "upnpdescgen.h"
@@ -58,6 +59,7 @@
 #include "options.h"
 #include "minissdp.h"
 #include "upnpredirect.h"
+#include "upnppinhole.h"
 #include "miniupnpdtypes.h"
 #include "daemonize.h"
 #include "upnpevents.h"
@@ -69,6 +71,11 @@
 #ifdef USE_IFACEWATCHER
 #include "ifacewatcher.h"
 #endif
+#ifdef ENABLE_6FC_SERVICE
+#ifdef USE_NETFILTER
+void init_iptpinhole(void);
+#endif
+#endif
 
 #ifndef DEFAULT_CONFIG
 #define DEFAULT_CONFIG "/etc/miniupnpd.conf"
@@ -93,8 +100,8 @@ int get_udp_dst_port (char *payload);
 #endif
 
 /* variables used by signals */
-static volatile int quitting = 0;
-volatile int should_send_public_address_change_notif = 0;
+static volatile sig_atomic_t quitting = 0;
+volatile sig_atomic_t should_send_public_address_change_notif = 0;
 
 /* OpenAndConfHTTPSocket() :
  * setup the socket used to handle incoming HTTP connections. */
@@ -416,10 +423,16 @@ write_command_line(int fd, int argc, char * * argv)
 static void
 sigterm(int sig)
 {
-	/*int save_errno = errno;*/
-	signal(sig, SIG_IGN);	/* Ignore this signal while we are quitting */
+	UNUSED(sig);
+	/*int save_errno = errno; */
+	/*signal(sig, SIG_IGN);*/	/* Ignore this signal while we are quitting */
+	/* Note : isn't it useless ? */
 
+#if 0
+	/* calling syslog() is forbidden in signal handler according to
+	 * signal(3) */
 	syslog(LOG_NOTICE, "received signal %d, good-bye", sig);
+#endif
 
 	quitting = 1;
 	/*errno = save_errno;*/
@@ -429,7 +442,12 @@ sigterm(int sig)
 static void
 sigusr1(int sig)
 {
+	UNUSED(sig);
+#if 0
+	/* calling syslog() is forbidden in signal handler according to
+	 * signal(3) */
 	syslog(LOG_INFO, "received signal %d, public ip address change", sig);
+#endif
 
 	should_send_public_address_change_notif = 1;
 }
@@ -519,6 +537,8 @@ struct runtime_vars {
  * external interface associated with the lan subnet follows.
  * ex : 192.168.1.1/24 81.21.41.11
  *
+ * Can also use the interface name (ie eth0)
+ *
  * return value :
  *    0 : ok
  *   -1 : error */
@@ -529,14 +549,26 @@ parselanaddr(struct lan_addr_s * lan_addr, const char * str)
 	int n;
 	char tmp[16];
 
+	memset(lan_addr, 0, sizeof(struct lan_addr_s));
 	p = str;
 	while(*p && *p != '/' && !isspace(*p))
 		p++;
 	n = p - str;
-	if(n>15)
-		goto parselan_error;
-	memcpy(lan_addr->str, str, n);
-	lan_addr->str[n] = '\0';
+	if(!isdigit(str[0]) && n < (int)sizeof(lan_addr->ifname))
+	{
+		/* not starting with a digit : suppose it is an interface name */
+		memcpy(lan_addr->ifname, str, n);
+		lan_addr->ifname[n] = '\0';
+		if(getifaddr(lan_addr->ifname, lan_addr->str, sizeof(lan_addr->str)) < 0)
+			goto parselan_error;
+	}
+	else
+	{
+		if(n>15)
+			goto parselan_error;
+		memcpy(lan_addr->str, str, n);
+		lan_addr->str[n] = '\0';
+	}
 	if(!inet_aton(lan_addr->str, &lan_addr->addr))
 		goto parselan_error;
 	if(*p == '/')
@@ -587,10 +619,20 @@ parselanaddr(struct lan_addr_s * lan_addr, const char * str)
 			}
 		}
 	}
+#endif
+#ifdef ENABLE_IPV6
+	if(lan_addr->ifname[0] != '\0')
+	{
+		lan_addr->index = if_nametoindex(lan_addr->ifname);
+		if(lan_addr->index == 0)
+			fprintf(stderr, "Cannot get index for network interface %s",
+			        lan_addr->ifname);
+	}
 #endif
 	return 0;
 parselan_error:
-	fprintf(stderr, "Error parsing address/mask : %s\n", str);
+	fprintf(stderr, "Error parsing address/mask (or interface name) : %s\n",
+	        str);
 	return -1;
 }
 
@@ -654,7 +696,7 @@ init(int argc, char * * argv, struct runtime_vars * v)
 	}
 	else
 	{
-		for(i=0; i<num_options; i++)
+		for(i=0; i<(int)num_options; i++)
 		{
 			switch(ary_options[i].id)
 			{
@@ -1011,23 +1053,23 @@ init(int argc, char * * argv, struct runtime_vars * v)
 	memset(&sa, 0, sizeof(struct sigaction));
 	sa.sa_handler = sigterm;
 
-	if (sigaction(SIGTERM, &sa, NULL))
+	if(sigaction(SIGTERM, &sa, NULL) < 0)
 	{
 		syslog(LOG_ERR, "Failed to set %s handler. EXITING", "SIGTERM");
 		return 1;
 	}
-	if (sigaction(SIGINT, &sa, NULL))
+	if(sigaction(SIGINT, &sa, NULL) < 0)
 	{
 		syslog(LOG_ERR, "Failed to set %s handler. EXITING", "SIGINT");
 		return 1;
 	}
-
-	if(signal(SIGPIPE, SIG_IGN) == SIG_ERR) {
+	sa.sa_handler = SIG_IGN;
+	if(sigaction(SIGPIPE, &sa, NULL) < 0)
+	{
 		syslog(LOG_ERR, "Failed to ignore SIGPIPE signals");
 	}
-
 	sa.sa_handler = sigusr1;
-	if (sigaction(SIGUSR1, &sa, NULL))
+	if(sigaction(SIGUSR1, &sa, NULL) < 0)
 	{
 		syslog(LOG_NOTICE, "Failed to set %s handler", "SIGUSR1");
 	}
@@ -1037,8 +1079,14 @@ init(int argc, char * * argv, struct runtime_vars * v)
 		syslog(LOG_ERR, "Failed to init redirection engine. EXITING");
 		return 1;
 	}
+#ifdef ENABLE_6FC_SERVICE
+#ifdef USE_NETFILTER
+	init_iptpinhole();
+#endif
+#endif
 
-	writepidfile(pidfilename, pid);
+	if(writepidfile(pidfilename, pid) < 0)
+		pidfilename = NULL;
 
 #ifdef ENABLE_LEASEFILE
 	/*remove(lease_file);*/
@@ -1133,7 +1181,9 @@ main(int argc, char * * argv)
 	struct rule_state * rule_list = 0;
 	struct timeval checktime = {0, 0};
 	struct lan_addr_s * lan_addr;
-	syslog(LOG_INFO, "SNet version started");
+#ifdef ENABLE_6FC_SERVICE
+	unsigned int next_pinhole_ts;
+#endif
 
 	if(init(argc, argv, &v) != 0)
 		return 1;
@@ -1142,7 +1192,12 @@ main(int argc, char * * argv)
 	for(lan_addr = lan_addrs.lh_first; lan_addr != NULL; lan_addr = lan_addr->list.le_next)
 		addr_count++;
 	if(addr_count > 0) {
+#ifndef ENABLE_IPV6
 		snotify = calloc(addr_count, sizeof(int));
+#else
+		/* one for IPv4, one for IPv6 */
+		snotify = calloc(addr_count * 2, sizeof(int));
+#endif
 	}
 #ifdef ENABLE_NATPMP
 	if(addr_count > 0) {
@@ -1268,6 +1323,7 @@ main(int argc, char * * argv)
 		}
 	}
 #endif
+
 	/* main loop */
 	while(!quitting)
 	{
@@ -1347,13 +1403,13 @@ main(int argc, char * * argv)
 		/* Remove expired port mappings, based on UPnP IGD LeaseDuration
 		 * or NAT-PMP lifetime) */
 		if(nextruletoclean_timestamp
-		  && (timeofday.tv_sec >= nextruletoclean_timestamp))
+		  && ((unsigned int)timeofday.tv_sec >= nextruletoclean_timestamp))
 		{
 			syslog(LOG_DEBUG, "cleaning expired Port Mappings");
 			get_upnp_rules_state_list(0);
 		}
 		if(nextruletoclean_timestamp
-		  && timeout.tv_sec >= (nextruletoclean_timestamp - timeofday.tv_sec))
+		  && ((unsigned int)timeout.tv_sec >= (nextruletoclean_timestamp - timeofday.tv_sec)))
 		{
 			timeout.tv_sec = nextruletoclean_timestamp - timeofday.tv_sec;
 			timeout.tv_usec = 0;
@@ -1381,6 +1437,16 @@ main(int argc, char * * argv)
 			timeout.tv_usec = 0;
 		}
 #endif
+#endif
+#ifdef ENABLE_6FC_SERVICE
+		/* Clean up expired IPv6 PinHoles */
+		next_pinhole_ts = 0;
+		upnp_clean_expired_pinholes(&next_pinhole_ts);
+		if(next_pinhole_ts &&
+		   timeout.tv_sec >= (int)(next_pinhole_ts - timeofday.tv_sec)) {
+			timeout.tv_sec = next_pinhole_ts - timeofday.tv_sec;
+			timeout.tv_usec = 0;
+		}
 #endif
 
 		/* select open sockets (SSDP, HTTP listen, and all HTTP soap sockets) */
@@ -1700,18 +1766,25 @@ shutdown:
 	}
 #endif
 
-	/*if(SendSSDPGoodbye(snotify, v.n_lan_addr) < 0)*/
 	if (GETFLAG(ENABLEUPNPMASK))
 	{
+#ifndef ENABLE_IPV6
 		if(SendSSDPGoodbye(snotify, addr_count) < 0)
+#else
+		if(SendSSDPGoodbye(snotify, addr_count * 2) < 0)
+#endif
 		{
 			syslog(LOG_ERR, "Failed to broadcast good-bye notifications");
 		}
-		for(i=0; i<addr_count; i++)
+#ifndef ENABLE_IPV6
+		for(i = 0; i < addr_count; i++)
+#else
+		for(i = 0; i < addr_count * 2; i++)
+#endif
 			close(snotify[i]);
 	}
 
-	if(unlink(pidfilename) < 0)
+	if(pidfilename && (unlink(pidfilename) < 0))
 	{
 		syslog(LOG_ERR, "Failed to remove pidfile %s: %m", pidfilename);
 	}
