@@ -1,7 +1,7 @@
-/* $Id: miniupnpdtypes.h,v 1.3 2011/05/13 13:56:18 nanard Exp $ */
+/* $Id: miniupnpdtypes.h,v 1.4 2012/04/06 15:27:21 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2007 Thomas Bernard
+ * (c) 2006-2012 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 #ifndef __MINIUPNPDTYPES_H__
@@ -9,11 +9,16 @@
 
 #include "config.h"
 #include <netinet/in.h>
+#include <net/if.h>
 #include <sys/queue.h>
 
 /* structure and list for storing lan addresses
  * with ascii representation and mask */
 struct lan_addr_s {
+	char ifname[IFNAMSIZ];	/* example: eth0 */
+#ifdef ENABLE_IPV6
+	unsigned int index;		/* use if_nametoindex() */
+#endif
 	char str[16];	/* example: 192.168.0.1 */
 	struct in_addr addr, mask;	/* ip/mask */
 #ifdef MULTIPLE_EXTERNAL_IP
