@@ -1,4 +1,4 @@
-/* $Id: natpmp.c,v 1.29 2012/03/05 20:36:17 nanard Exp $ */
+/* $Id: natpmp.c,v 1.32 2012/05/27 22:36:03 nanard Exp $ */
 /* MiniUPnP project
  * (c) 2007-2012 Thomas Bernard
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
@@ -14,25 +14,39 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+
+#include "macros.h"
 #include "config.h"
 #include "natpmp.h"
 #include "upnpglobalvars.h"
 #include "getifaddr.h"
 #include "upnpredirect.h"
 #include "commonrdr.h"
+#include "upnputils.h"
 
 #ifdef ENABLE_NATPMP
 
 int OpenAndConfNATPMPSocket(in_addr_t addr)
 {
 	int snatpmp;
+	int i = 1;
 	snatpmp = socket(PF_INET, SOCK_DGRAM, 0/*IPPROTO_UDP*/);
 	if(snatpmp<0)
 	{
-		syslog(LOG_ERR, "socket(natpmp): %m");
+		syslog(LOG_ERR, "%s: socket(natpmp): %m",
+		       "OpenAndConfNATPMPSocket");
 		return -1;
 	}
-	else
+	if(setsockopt(snatpmp, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i)) < 0)
+	{
+		syslog(LOG_WARNING, "%s: setsockopt(natpmp, SO_REUSEADDR): %m",
+		       "OpenAndConfNATPMPSocket");
+	}
+	if(!set_non_blocking(snatpmp))
+	{
+		syslog(LOG_WARNING, "%s: set_non_blocking(): %m",
+		       "OpenAndConfNATPMPSocket");
+	}
 	{
 		struct sockaddr_in natpmp_addr;
 		memset(&natpmp_addr, 0, sizeof(natpmp_addr));
@@ -74,6 +88,7 @@ static void FillPublicAddressResponse(unsigned char * resp, in_addr_t senderaddr
 {
 #ifndef MULTIPLE_EXTERNAL_IP
 	char tmp[16];
+	UNUSED(senderaddr);
 
 	if(use_ext_ip_addr) {
         inet_pton(AF_INET, use_ext_ip_addr, resp+8);
@@ -91,6 +106,7 @@ static void FillPublicAddressResponse(unsigned char * resp, in_addr_t senderaddr
 	}
 #else
 	struct lan_addr_s * lan_addr;
+
 	for(lan_addr = lan_addrs.lh_first; lan_addr != NULL; lan_addr = lan_addr->list.le_next) {
 		if( (senderaddr & lan_addr->mask.s_addr)
 		   == (lan_addr->addr.s_addr & lan_addr->mask.s_addr)) {
