@@ -1,4 +1,4 @@
-/* $Id: iptcrdr.c,v 1.48 2012/03/05 20:36:19 nanard Exp $ */
+/* $Id: iptcrdr.c,v 1.49 2012/04/30 21:08:02 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2011 Thomas Bernard
@@ -56,6 +56,8 @@
 #define IPT_ALIGN XT_ALIGN
 #endif
 
+#include "../macros.h"
+#include "../config.h"
 #include "iptcrdr.h"
 #include "../upnpglobalvars.h"
 
@@ -204,7 +206,10 @@ add_redirect_rule2(const char * ifname,
                    const char * iaddr, unsigned short iport, int proto,
 				   const char * desc, unsigned int timestamp)
 {
-	int r = addnatrule(proto, eport, iaddr, iport, rhost);
+	int r;
+	UNUSED(ifname);
+
+	r = addnatrule(proto, eport, iaddr, iport, rhost);
 	if(r >= 0)
 		add_redirect_desc(eport, proto, desc, timestamp);
 	return r;
@@ -216,6 +221,10 @@ add_filter_rule2(const char * ifname,
                  unsigned short eport, unsigned short iport,
                  int proto, const char * desc)
 {
+	UNUSED(ifname);
+	UNUSED(eport);
+	UNUSED(desc);
+
 	return add_filter_rule(proto, rhost, iaddr, iport);
 }
 
@@ -235,6 +244,7 @@ get_redirect_rule(const char * ifname, unsigned short eport, int proto,
 	const struct ipt_entry_target * target;
 	const struct ip_nat_multi_range * mr;
 	const struct ipt_entry_match *match;
+	UNUSED(ifname);
 
 	h = iptc_init("nat");
 	if(!h)
@@ -332,6 +342,7 @@ get_redirect_rule_by_index(int index,
 	const struct ipt_entry_target * target;
 	const struct ip_nat_multi_range * mr;
 	const struct ipt_entry_match *match;
+	UNUSED(ifname);
 
 	h = iptc_init("nat");
 	if(!h)
