@@ -1,7 +1,7 @@
-/* $Id: upnphttp.h,v 1.28 2012/03/05 20:36:18 nanard Exp $ */
+/* $Id: upnphttp.h,v 1.30 2012/04/25 22:26:05 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2011 Thomas Bernard
+ * (c) 2006-2012 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -13,8 +13,16 @@
 
 #include "config.h"
 
+#if 0
+/* according to "UPnP Device Architecture 1.0" */
+#define UPNP_VERSION_STRING "UPnP/1.0"
+#else
+/* according to "UPnP Device Architecture 1.1" */
+#define UPNP_VERSION_STRING "UPnP/1.1"
+#endif
+
 /* server: HTTP header returned in all HTTP responses : */
-#define MINIUPNPD_SERVER_STRING	OS_VERSION " UPnP/1.0 MiniUPnPd/" MINIUPNPD_VERSION
+#define MINIUPNPD_SERVER_STRING	OS_VERSION " " UPNP_VERSION_STRING " MiniUPnPd/" MINIUPNPD_VERSION
 
 /*
  states :
@@ -53,14 +61,15 @@ struct upnphttp {
 	int req_contentlen;
 	int req_contentoff;     /* header length */
 	enum httpCommands req_command;
-	const char * req_soapAction;
+	int req_soapActionOff;
 	int req_soapActionLen;
 #ifdef ENABLE_EVENTS
-	const char * req_Callback;	/* For SUBSCRIBE */
+	int req_CallbackOff;	/* For SUBSCRIBE */
 	int req_CallbackLen;
 	int req_Timeout;
-	const char * req_SID;		/* For UNSUBSCRIBE */
+	int req_SIDOff;		/* For UNSUBSCRIBE */
 	int req_SIDLen;
+	const char * res_SID;
 #endif
 	int respflags;				/* see FLAG_* constants below */
 	/* response */
