@@ -1,4 +1,4 @@
-/* $Id: upnpsoap.c,v 1.91 2012/03/05 20:36:18 nanard Exp $ */
+/* $Id: upnpsoap.c,v 1.110 2012/05/24 16:51:09 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2012 Thomas Bernard
@@ -8,6 +8,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <sys/socket.h>
 #include <unistd.h>
 #include <syslog.h>
@@ -16,12 +17,14 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 
+#include "macros.h"
 #include "config.h"
 #include "upnpglobalvars.h"
 #include "upnphttp.h"
 #include "upnpsoap.h"
 #include "upnpreplyparse.h"
 #include "upnpredirect.h"
+#include "upnppinhole.h"
 #include "getifaddr.h"
 #include "getifstats.h"
 #include "getconnstatus.h"
@@ -65,6 +68,8 @@ GetConnectionTypeInfo(struct upnphttp * h, const char * action)
 		"<NewConnectionType>IP_Routed</NewConnectionType>"
 		"<NewPossibleConnectionTypes>IP_Routed</NewPossibleConnectionTypes>"
 		"</u:GetConnectionTypeInfoResponse>";
+	UNUSED(action);
+
 	BuildSendAndCloseSoapResp(h, resp, sizeof(resp)-1);
 }
 
@@ -232,6 +237,7 @@ GetNATRSIPStatus(struct upnphttp * h, const char * action)
 		"<NewRSIPAvailable>0</NewRSIPAvailable>"
 		"<NewNATEnabled>1</NewNATEnabled>"
 		"</u:GetNATRSIPStatusResponse>";
+	UNUSED(action);
 	/* 2.2.9. RSIPAvailable
 	 * This variable indicates if Realm-specific IP (RSIP) is available
 	 * as a feature on the InternetGatewayDevice. RSIP is being defined
@@ -400,7 +406,8 @@ AddPortMapping(struct upnphttp * h, const char * action)
 #endif
 
 	syslog(LOG_INFO, "%s: ext port %hu to %s:%hu protocol %s for: %s leaseduration=%u rhost=%s",
-	       action, eport, int_ip, iport, protocol, desc, leaseduration, r_host);
+	       action, eport, int_ip, iport, protocol, desc, leaseduration,
+	       r_host ? r_host : "NULL");
 
 	r = upnp_redirect(r_host, eport, int_ip, iport, protocol, desc, leaseduration);
 
@@ -633,7 +640,8 @@ GetSpecificPortMappingEntry(struct upnphttp * h, const char * action)
 	{
 		syslog(LOG_INFO, "%s: rhost='%s' %s %s found => %s:%u desc='%s'",
 		       action,
-		       r_host, ext_port, protocol, int_ip, (unsigned int)iport, desc);
+		       r_host ? r_host : "NULL", ext_port, protocol, int_ip,
+		       (unsigned int)iport, desc);
 		bodylen = snprintf(body, sizeof(body), resp,
 				action, SERVICE_TYPE_WANIPC,
 				(unsigned int)iport, int_ip, desc, leaseduration,
@@ -719,6 +727,7 @@ DeletePortMappingRange(struct upnphttp * h, const char * action)
 	int manage;
 	unsigned short * port_list;
 	unsigned int i, number = 0;
+	UNUSED(action);
 
 	ParseNameValue(h->req_buf + h->req_contentoff, h->req_contentlen, &data);
 	startport = (unsigned short)atoi(GetValueFromNameValueList(&data, "NewStartPort"));
@@ -911,6 +920,11 @@ http://www.upnp.org/schemas/gw/WANIPConnection-v2.xsd">
 	}
 	bodylen = snprintf(body, bodyalloc, resp_start,
 	              action, SERVICE_TYPE_WANIPC);
+	if(bodylen < 0)
+	{
+		SoapError(h, 501, "ActionFailed");
+		return;
+	}
 	memcpy(body+bodylen, list_start, sizeof(list_start));
 	bodylen += (sizeof(list_start) - 1);
 
@@ -920,7 +934,7 @@ http://www.upnp.org/schemas/gw/WANIPConnection-v2.xsd">
 	for(i = 0; number > 0 && i < list_size; i++)
 	{
 		/* have a margin of 1024 bytes to store the new entry */
-		if(bodylen + 1024 > bodyalloc)
+		if((unsigned int)bodylen + 1024 > bodyalloc)
 		{
 			bodyalloc += 4096;
 			body = realloc(body, bodyalloc);
@@ -1005,6 +1019,7 @@ SetConnectionType(struct upnphttp * h, const char * action)
 {
 	const char * connection_type;
 	struct NameValueParserData data;
+	UNUSED(action);
 
 	ParseNameValue(h->req_buf + h->req_contentoff, h->req_contentlen, &data);
 	connection_type = GetValueFromNameValueList(&data, "NewConnectionType");
@@ -1018,6 +1033,7 @@ SetConnectionType(struct upnphttp * h, const char * action)
 static void
 RequestConnection(struct upnphttp * h, const char * action)
 {
+	UNUSED(action);
 	SoapError(h, 606, "Action not authorized");
 }
 
@@ -1025,6 +1041,7 @@ RequestConnection(struct upnphttp * h, const char * action)
 static void
 ForceTermination(struct upnphttp * h, const char * action)
 {
+	UNUSED(action);
 	SoapError(h, 606, "Action not authorized");
 }
 
@@ -1132,7 +1149,7 @@ CheckStatus(struct upnphttp * h)
 {
 	if (!ipv6fc_firewall_enabled)
 	{
-		SoapError(h, 702, "FirewallDisabed");
+		SoapError(h, 702, "FirewallDisabled");
 		return 0;
 	}
 	else if(!ipv6fc_inbound_pinhole_allowed)
@@ -1144,62 +1161,6 @@ CheckStatus(struct upnphttp * h)
 		return 1;
 }
 
-static int
-DataVerification(struct upnphttp * h, char * int_ip, unsigned short * int_port, const char * protocol, char * leaseTime)
-{
-	/* **  Internal IP can't be wildcarded */
-	if (!int_ip)
-	{
-		SoapError(h, 708, "WildCardNotPermittedInSrcIP");
-		return 0;
-	}
-
-	if (!strchr(int_ip, ':'))
-	{
-		SoapError(h, 402, "Invalid Args");
-		return 0;
-	}
-
-	/* ** Internal port can't be wilcarded.  */
-/*	printf("\tint_port: *%d*\n", *int_port); */
-	if (*int_port == 0)
-	{
-		SoapError(h, 706, "InternalPortWilcardingNotAllowed");
-		return 0;
-	}
-
-	/* ** Protocol can't be wilcarded and can't be an unknown port
-	 * (here deal with only UDP, TCP, UDPLITE) */
-/*	printf("\tprotocol: *%s*\n", protocol); */
-	if (atoi(protocol) == 65535)
-	{
-		SoapError(h, 707, "ProtocolWilcardingNotAllowed");
-		return 0;
-	}
-	else if (atoi(protocol) != IPPROTO_UDP
-	        && atoi(protocol) != IPPROTO_TCP
-#ifdef IPPROTO_UDPITE
-	        && atoi(protocol) != IPPROTO_UDPLITE
-#endif
-	        )
-	{
-		SoapError(h, 705, "ProtocolNotSupported");
-		return 0;
-	}
-
-	/* ** Lease Time can't be wilcarded nor >86400. */
-/*	printf("\tlease time: %s\n", leaseTime); */
-	if(!leaseTime || !atoi(leaseTime) || atoi(leaseTime)>86400)
-	{
-		/* lease duration is never infinite, nor wilcarded. In this case, use default value */
-		syslog(LOG_WARNING, "LeaseTime=%s not supported, (ip=%s)", leaseTime, int_ip);
-		SoapError(h, 402, "Invalid Args");
-		return 0;
-	}
-
-	return 1;
-}
-
 #if 0
 static int connecthostport(const char * host, unsigned short port, char * result)
 {
@@ -1256,34 +1217,29 @@ static int connecthostport(const char * host, unsigned short port, char * result
 
 /* Check the security policy right */
 static int
-PinholeVerification(struct upnphttp * h, char * int_ip, unsigned short * int_port)
+PinholeVerification(struct upnphttp * h, char * int_ip, unsigned short int_port)
 {
 	int n;
+	char senderAddr[INET6_ADDRSTRLEN]="";
+	struct addrinfo hints, *ai, *p;
+	struct in6_addr result_ip;
+
 	/* Pinhole InternalClient address must correspond to the action sender */
 	syslog(LOG_INFO, "Checking internal IP@ and port (Security policy purpose)");
-	char senderAddr[INET6_ADDRSTRLEN]="";
-	//char str[INET6_ADDRSTRLEN]="";
-	//connecthostport(int_ip, *int_port, str);
-	//printf("int_ip: %s / str: %s\n", int_ip, str);
-
-	struct addrinfo hints, *ai, *p;
-	struct in6_addr result_ip;/*unsigned char result_ip[16];*/ /* inet_pton() */ //IPv6 Modification
 
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_family = AF_UNSPEC;
 
 	/* if ip not valid assume hostname and convert */
-	if (inet_pton(AF_INET6, int_ip, &result_ip) <= 0) //IPv6 Modification
+	if (inet_pton(AF_INET6, int_ip, &result_ip) <= 0)
 	{
-
-		n = getaddrinfo(int_ip, NULL, &hints, &ai);//hp = gethostbyname(int_ip);
-		if(!n && ai->ai_family == AF_INET6) //IPv6 Modification
+		n = getaddrinfo(int_ip, NULL, &hints, &ai);
+		if(!n && ai->ai_family == AF_INET6)
 		{
-			for(p = ai; p; p = p->ai_next)//ptr = hp->h_addr_list; ptr && *ptr; ptr++)
-		   	{
-				inet_ntop(AF_INET6, (struct in6_addr *) p, int_ip, sizeof(struct in6_addr)); ///IPv6 Modification
+			for(p = ai; p; p = p->ai_next)
+			{
+				inet_ntop(AF_INET6, (struct in6_addr *) p, int_ip, sizeof(struct in6_addr));
 				result_ip = *((struct in6_addr *) p);
-				fprintf(stderr, "upnpsoap / AddPinhole: assuming int addr = %s", int_ip);
 				/* TODO : deal with more than one ip per hostname */
 				break;
 			}
@@ -1297,9 +1253,8 @@ PinholeVerification(struct upnphttp * h, char * int_ip, unsigned short * int_por
         freeaddrinfo(p);
 	}
 
-	if(inet_ntop(AF_INET6, &(h->clientaddr_v6), senderAddr, INET6_ADDRSTRLEN)<=0)
+	if(inet_ntop(AF_INET6, &(h->clientaddr_v6), senderAddr, INET6_ADDRSTRLEN) == NULL)
 	{
-		//printf("Failed to inet_ntop\n");
 		syslog(LOG_ERR, "inet_ntop: %m");
 	}
 #ifdef DEBUG
@@ -1315,7 +1270,7 @@ PinholeVerification(struct upnphttp * h, char * int_ip, unsigned short * int_por
 	}
 
 	/* Pinhole InternalPort must be greater than or equal to 1024 */
-	if (*int_port < 1024)
+	if (int_port < 1024)
 	{
 		syslog(LOG_INFO, "Client %s tried to access pinhole with port < 1024 and is not authorized to do it",
 		       senderAddr);
@@ -1340,6 +1295,9 @@ AddPinhole(struct upnphttp * h, const char * action)
 	char * rem_host, * rem_port, * int_ip, * int_port, * protocol, * leaseTime;
 	int uid = 0;
 	unsigned short iport, rport;
+	int ltime;
+	long proto;
+	char rem_ip[INET6_ADDRSTRLEN];
 
 	if(CheckStatus(h)==0)
 		return;
@@ -1352,36 +1310,117 @@ AddPinhole(struct upnphttp * h, const char * action)
 	protocol = GetValueFromNameValueList(&data, "Protocol");
 	leaseTime = GetValueFromNameValueList(&data, "LeaseTime");
 
-	rport = (unsigned short)atoi(rem_port);
-	iport = (unsigned short)atoi(int_port);
-
-	/* **  As there is no security policy, InternalClient must be equal
-	 * to the CP's IP address. */
-	if(DataVerification(h, int_ip, &iport, protocol, leaseTime) == 0
-	   || PinholeVerification(h, int_ip, &iport) <= 0)
+	rport = (unsigned short)(rem_port ? atoi(rem_port) : 0);
+	iport = (unsigned short)(int_port ? atoi(int_port) : 0);
+	ltime = leaseTime ? atoi(leaseTime) : -1;
+	errno = 0;
+	proto = protocol ? strtol(protocol, NULL, 0) : -1;
+	if(errno != 0 || proto > 65535 || proto < 0)
 	{
-		ClearNameValueList(&data);
-		return ;
-	}
-
-	/* ** RemoteHost can be wilcarded or an IDN. */
-	/*printf("\trem_host: %s\n", rem_host);*/
-	if (rem_host!=NULL && !strchr(rem_host, ':'))
-	{
-		ClearNameValueList(&data);
 		SoapError(h, 402, "Invalid Args");
-		return;
+		goto clear_and_exit;
+	}
+	if(iport == 0)
+	{
+		SoapError(h, 706, "InternalPortWilcardingNotAllowed");
+		goto clear_and_exit;
 	}
-	/*printf("\tAddr check passed.\n");*/
 
-	syslog(LOG_INFO, "%s: (inbound) from [%s]:%hu to [%s]:%hu with protocol %s during %ssec", action, rem_host?rem_host:"anywhere", rport, int_ip, iport, protocol, leaseTime);
+	/* In particular, [IGD2] RECOMMENDS that unauthenticated and
+	 * unauthorized control points are only allowed to invoke
+	 * this action with:
+	 * - InternalPort value greater than or equal to 1024,
+	 * - InternalClient value equals to the control point's IP address.
+	 * It is REQUIRED that InternalClient cannot be one of IPv6
+	 * addresses used by the gateway. */
+	if(!int_ip || 0 == strlen(int_ip) || 0 == strcmp(int_ip, "*"))
+	{
+		SoapError(h, 708, "WildCardNotPermittedInSrcIP");
+		goto clear_and_exit;
+	}
+	/* I guess it is useless to convert int_ip to literal ipv6 address */
+	/* rem_host should be converted to literal ipv6 : */
+	if(rem_host)
+	{
+		struct addrinfo *ai, *p;
+		struct addrinfo hints;
+		int err;
+		memset(&hints, 0, sizeof(struct addrinfo));
+		hints.ai_family = AF_INET6;
+		/*hints.ai_flags = */
+		/* hints.ai_protocol = proto; */
+		err = getaddrinfo(rem_host, rem_port, &hints, &ai);
+		if(err == 0)
+		{
+			/* take the 1st IPv6 address */
+			for(p = ai; p; p = p->ai_next)
+			{
+				if(p->ai_family == AF_INET6)
+				{
+					inet_ntop(AF_INET6,
+					          &(((struct sockaddr_in6 *)p->ai_addr)->sin6_addr),
+					          rem_ip, sizeof(rem_ip));
+					syslog(LOG_INFO, "resolved '%s' to '%s'", rem_host, rem_ip);
+					rem_host = rem_ip;
+					break;
+				}
+			}
+			freeaddrinfo(ai);
+		}
+		else
+		{
+			syslog(LOG_WARNING, "AddPinhole : getaddrinfo(%s) : %s",
+			       rem_host, gai_strerror(err));
+#if 0
+			SoapError(h, 402, "Invalid Args");
+			goto clear_and_exit;
+#endif
+		}
+	}
 
-	r = upnp_add_inboundpinhole(rem_host, rport, int_ip, iport, protocol, leaseTime, &uid);
+	if(proto == 65535)
+	{
+		SoapError(h, 707, "ProtocolWilcardingNotAllowed");
+		goto clear_and_exit;
+	}
+	if(proto != IPPROTO_UDP && proto != IPPROTO_TCP
+#ifdef IPPROTO_UDPITE
+	   && atoi(protocol) != IPPROTO_UDPLITE
+#endif
+	  )
+	{
+		SoapError(h, 705, "ProtocolNotSupported");
+		goto clear_and_exit;
+	}
+	if(ltime < 1 || ltime > 86400)
+	{
+		syslog(LOG_WARNING, "%s: LeaseTime=%d not supported, (ip=%s)",
+		       action, ltime, int_ip);
+		SoapError(h, 402, "Invalid Args");
+		goto clear_and_exit;
+	}
+
+	if(PinholeVerification(h, int_ip, iport) <= 0)
+		goto clear_and_exit;
+
+	syslog(LOG_INFO, "%s: (inbound) from [%s]:%hu to [%s]:%hu with proto %ld during %d sec",
+	       action, rem_host?rem_host:"any",
+	       rport, int_ip, iport,
+	       proto, ltime);
+
+	/* In cases where the RemoteHost, RemotePort, InternalPort,
+	 * InternalClient and Protocol are the same than an existing pinhole,
+	 * but LeaseTime is different, the device MUST extend the existing
+	 * pinhole's lease time and return the UniqueID of the existing pinhole. */
+	r = upnp_add_inboundpinhole(rem_host, rport, int_ip, iport, proto, ltime, &uid);
 
 	switch(r)
 	{
 		case 1:	        /* success */
-			bodylen = snprintf(body, sizeof(body), resp, action, "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1", uid, action);
+			bodylen = snprintf(body, sizeof(body),
+			                   resp, action,
+			                   "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1",
+			                   uid, action);
 			BuildSendAndCloseSoapResp(h, body, bodylen);
 			break;
 		case -1: 	/* not permitted */
@@ -1391,74 +1430,85 @@ AddPinhole(struct upnphttp * h, const char * action)
 			SoapError(h, 501, "ActionFailed");
 			break;
 	}
+	/* 606 Action not authorized
+	 * 701 PinholeSpaceExhausted
+	 * 702 FirewallDisabled
+	 * 703 InboundPinholeNotAllowed
+	 * 705 ProtocolNotSupported
+	 * 706 InternalPortWildcardingNotAllowed
+	 * 707 ProtocolWildcardingNotAllowed
+	 * 708 WildCardNotPermittedInSrcIP */
+clear_and_exit:
 	ClearNameValueList(&data);
 }
 
 static void
 UpdatePinhole(struct upnphttp * h, const char * action)
 {
-	int r, n;
 	static const char resp[] =
 		"<u:UpdatePinholeResponse "
 		"xmlns:u=\"urn:schemas-upnp-org:service:WANIPv6FirewallControl:1\">"
 		"</u:UpdatePinholeResponse>";
 	struct NameValueParserData data;
-	const char * uid, * leaseTime;
-	char iaddr[40], proto[6], lt[12];
+	const char * uid_str, * leaseTime;
+	char iaddr[INET6_ADDRSTRLEN];
 	unsigned short iport;
+	int ltime;
+	int uid;
+	int n;
 
 	if(CheckStatus(h)==0)
 		return;
 
 	ParseNameValue(h->req_buf + h->req_contentoff, h->req_contentlen, &data);
-	uid = GetValueFromNameValueList(&data, "UniqueID");
+	uid_str = GetValueFromNameValueList(&data, "UniqueID");
 	leaseTime = GetValueFromNameValueList(&data, "NewLeaseTime");
+	uid = uid_str ? atoi(uid_str) : -1;
+	ltime = leaseTime ? atoi(leaseTime) : -1;
+	ClearNameValueList(&data);
 
-	if(!uid || !leaseTime || !atoi(leaseTime) || atoi(leaseTime) > 86400)
+	if(uid < 0 || uid > 65535 || ltime <= 0 || ltime > 86400)
 	{
-		ClearNameValueList(&data);
 		SoapError(h, 402, "Invalid Args");
 		return;
 	}
 
-	/* Check that client is not deleting an pinhole
+	/* Check that client is not updating an pinhole
 	 * it doesn't have access to, because of its public access */
-	n = upnp_get_pinhole_info(0, 0, iaddr, &iport, proto, uid, lt);
-	if (n > 0)
+	n = upnp_get_pinhole_info(uid, NULL, 0, NULL,
+	                          iaddr, sizeof(iaddr), &iport,
+	                          NULL, NULL, NULL);
+	if (n >= 0)
 	{
-		if(PinholeVerification(h, iaddr, &iport)==0)
-		{
-			ClearNameValueList(&data);
-			return ;
-		}
+		if(PinholeVerification(h, iaddr, iport) <= 0)
+			return;
 	}
-
-	syslog(LOG_INFO, "%s: (inbound) updating lease duration to %s for pinhole with ID: %s", action, leaseTime, uid);
-
-	r = upnp_update_inboundpinhole(uid, leaseTime);
-
-	if(r < 0)
+	else if(n == -2)
 	{
-		if(r == -4 || r == -1)
-			SoapError(h, 704, "NoSuchEntry");
-		else
-			SoapError(h, 501, "ActionFailed");
+		SoapError(h, 704, "NoSuchEntry");
+		return;
 	}
 	else
 	{
-		BuildSendAndCloseSoapResp(h, resp, sizeof(resp)-1);
+		SoapError(h, 501, "ActionFailed");
+		return;
 	}
-	ClearNameValueList(&data);
+
+	syslog(LOG_INFO, "%s: (inbound) updating lease duration to %d for pinhole with ID: %d",
+	       action, ltime, uid);
+
+	n = upnp_update_inboundpinhole(uid, ltime);
+	if(n == -1)
+		SoapError(h, 704, "NoSuchEntry");
+	else if(n < 0)
+		SoapError(h, 501, "ActionFailed");
+	else
+		BuildSendAndCloseSoapResp(h, resp, sizeof(resp)-1);
 }
 
 static void
 GetOutboundPinholeTimeout(struct upnphttp * h, const char * action)
 {
-	if (!ipv6fc_firewall_enabled)
-	{
-		SoapError(h, 702, "FirewallDisabed");
-		return;
-	}
 	int r;
 
 	static const char resp[] =
@@ -1474,6 +1524,12 @@ GetOutboundPinholeTimeout(struct upnphttp * h, const char * action)
 	int opt=0, proto=0;
 	unsigned short iport, rport;
 
+	if (!ipv6fc_firewall_enabled)
+	{
+		SoapError(h, 702, "FirewallDisabled");
+		return;
+	}
+
 	ParseNameValue(h->req_buf + h->req_contentoff, h->req_contentlen, &data);
 	int_ip = GetValueFromNameValueList(&data, "InternalClient");
 	int_port = GetValueFromNameValueList(&data, "InternalPort");
@@ -1487,12 +1543,15 @@ GetOutboundPinholeTimeout(struct upnphttp * h, const char * action)
 
 	syslog(LOG_INFO, "%s: retrieving timeout for outbound pinhole from [%s]:%hu to [%s]:%hu protocol %s", action, int_ip, iport,rem_host, rport, protocol);
 
-	r = upnp_check_outbound_pinhole(proto, &opt);
+	/* TODO */
+	r = -1;/*upnp_check_outbound_pinhole(proto, &opt);*/
 
 	switch(r)
 	{
 		case 1:	/* success */
-			bodylen = snprintf(body, sizeof(body), resp, action, "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1", opt, action);
+			bodylen = snprintf(body, sizeof(body), resp,
+			                   action, "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1",
+			                   opt, action);
 			BuildSendAndCloseSoapResp(h, body, bodylen);
 			break;
 		case -5:	/* Protocol not supported */
@@ -1507,9 +1566,7 @@ GetOutboundPinholeTimeout(struct upnphttp * h, const char * action)
 static void
 DeletePinhole(struct upnphttp * h, const char * action)
 {
-	if(CheckStatus(h)==0)
-		return;
-	int r, n;
+	int n;
 
 	static const char resp[] =
 		"<u:DeletePinholeResponse "
@@ -1517,212 +1574,174 @@ DeletePinhole(struct upnphttp * h, const char * action)
 		"</u:DeletePinholeResponse>";
 
 	struct NameValueParserData data;
-	const char * uid;
-	char iaddr[40], proto[6], lt[12];
+	const char * uid_str;
+	char iaddr[INET6_ADDRSTRLEN];
+	int proto;
 	unsigned short iport;
+	unsigned int leasetime;
+	int uid;
+
+	if(CheckStatus(h)==0)
+		return;
 
 	ParseNameValue(h->req_buf + h->req_contentoff, h->req_contentlen, &data);
-	uid = GetValueFromNameValueList(&data, "UniqueID");
+	uid_str = GetValueFromNameValueList(&data, "UniqueID");
+	uid = uid_str ? atoi(uid_str) : -1;
+	ClearNameValueList(&data);
 
-	if(!uid)
+	if(uid < 0 || uid > 65535)
 	{
-		ClearNameValueList(&data);
 		SoapError(h, 402, "Invalid Args");
 		return;
 	}
 
 	/* Check that client is not deleting an pinhole
 	 * it doesn't have access to, because of its public access */
-	n = upnp_get_pinhole_info(0, 0, iaddr, &iport, proto, uid, lt);
-	if (n > 0)
+	n = upnp_get_pinhole_info(uid, NULL, 0, NULL,
+	                          iaddr, sizeof(iaddr), &iport,
+	                          &proto, &leasetime, NULL);
+	if (n >= 0)
 	{
-		if(PinholeVerification(h, iaddr, &iport)==0)
-		{
-			ClearNameValueList(&data);
-			return ;
-		}
+		if(PinholeVerification(h, iaddr, iport) <= 0)
+			return;
 	}
-
-	syslog(LOG_INFO, "%s: (inbound) delete pinhole with ID: %s", action, uid);
-
-	r = upnp_delete_inboundpinhole(uid);
-
-	if(r <= 0)
+	else if(n == -2)
 	{
-		syslog(LOG_INFO, "%s: (inbound) failed to remove pinhole with ID: %s", action, uid);
-		if(r==-4)
-			SoapError(h, 704, "NoSuchEntry");
-		else
-			SoapError(h, 501, "ActionFailed");
+		SoapError(h, 704, "NoSuchEntry");
+		return;
 	}
 	else
 	{
-		syslog(LOG_INFO, "%s: (inbound) pinhole successfully removed", action);
-		BuildSendAndCloseSoapResp(h, resp, sizeof(resp)-1);
+		SoapError(h, 501, "ActionFailed");
+		return;
 	}
-	ClearNameValueList(&data);
+
+	n = upnp_delete_inboundpinhole(uid);
+	if(n < 0)
+	{
+		syslog(LOG_INFO, "%s: (inbound) failed to remove pinhole with ID: %d",
+	           action, uid);
+		SoapError(h, 501, "ActionFailed");
+		return;
+	}
+	syslog(LOG_INFO, "%s: (inbound) pinhole with ID %d successfully removed",
+	       action, uid);
+	BuildSendAndCloseSoapResp(h, resp, sizeof(resp)-1);
 }
 
 static void
 CheckPinholeWorking(struct upnphttp * h, const char * action)
 {
-	if(CheckStatus(h)==0)
-		return;
-	int r, d;
-
 	static const char resp[] =
 		"<u:%sResponse "
 		"xmlns:u=\"%s\">"
 		"<IsWorking>%d</IsWorking>"
 		"</u:%sResponse>";
-
 	char body[512];
 	int bodylen;
+	int r;
 	struct NameValueParserData data;
-	const char * uid;
-	char eaddr[40], iaddr[40], proto[6], lt[12];
-	unsigned short eport, iport;
-	int isWorking = 0;
+	const char * uid_str;
+	int uid;
+	char iaddr[INET6_ADDRSTRLEN];
+	unsigned short iport;
+	unsigned int packets;
+
+	if(CheckStatus(h)==0)
+		return;
 
 	ParseNameValue(h->req_buf + h->req_contentoff, h->req_contentlen, &data);
-	uid = GetValueFromNameValueList(&data, "UniqueID");
+	uid_str = GetValueFromNameValueList(&data, "UniqueID");
+	uid = uid_str ? atoi(uid_str) : -1;
+	ClearNameValueList(&data);
 
-	if(!uid)
+	if(uid < 0 || uid > 65535)
 	{
-		ClearNameValueList(&data);
 		SoapError(h, 402, "Invalid Args");
 		return;
 	}
 
 	/* Check that client is not checking a pinhole
 	 * it doesn't have access to, because of its public access */
-	r = upnp_get_pinhole_info(eaddr, eport, iaddr, &iport, proto, uid, lt);
-	if (r > 0)
+	r = upnp_get_pinhole_info(uid,
+	                          NULL, 0, NULL,
+	                          iaddr, sizeof(iaddr), &iport,
+	                          NULL, NULL, &packets);
+	if (r >= 0)
 	{
-		if(PinholeVerification(h, iaddr, &iport)==0)
-		{
-			ClearNameValueList(&data);
+		if(PinholeVerification(h, iaddr, iport) <= 0)
 			return ;
-		}
-		else
+		if(packets == 0)
 		{
-			int rulenum_used, rulenum = 0;
-			d = upnp_check_pinhole_working(uid, eaddr, iaddr, &eport, &iport, proto, &rulenum_used);
-			if(d < 0)
-			{
-				if(d == -4)
-				{
-					syslog(LOG_INFO, "%s: rule for ID=%s, no trace found for this pinhole", action, uid);
-					SoapError(h, 709, "NoPacketSent");
-					ClearNameValueList(&data);
-					return ;
-				}
-				else
-				{
-				// d==-5 not same table // d==-6 not same chain // d==-7 not found a rule but policy traced
-					isWorking=0;
-					syslog(LOG_INFO, "%s: rule for ID=%s is not working, packet going through %s", action, uid, (d==-5)?"the wrong table":((d==-6)?"the wrong chain":"a chain policy"));
-					bodylen = snprintf(body, sizeof(body), resp,
-						action, "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1",
-						isWorking, action);
-					BuildSendAndCloseSoapResp(h, body, bodylen);
-				}
-			}
-			else
-			{
-				/*check_rule_from_file(uid, &rulenum);*/
-				if(rulenum_used == rulenum)
-				{
-					isWorking=1;
-					syslog(LOG_INFO, "%s: rule for ID=%s is working properly", action, uid);
-				}
-				else
-				{
-					isWorking=0;
-					syslog(LOG_INFO, "%s: rule for ID=%s is not working", action, uid);
-				}
-				bodylen = snprintf(body, sizeof(body), resp,
-						action, "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1",
-						isWorking, action);
-				BuildSendAndCloseSoapResp(h, body, bodylen);
-			}
+			SoapError(h, 709, "NoPacketSent");
+			return;
 		}
+		bodylen = snprintf(body, sizeof(body), resp,
+						action, "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1",
+						1, action);
+		BuildSendAndCloseSoapResp(h, body, bodylen);
 	}
-	else if(r == -4 || r == -1)
-	{
+	else if(r == -2)
 		SoapError(h, 704, "NoSuchEntry");
-	}
 	else
-	{
 		SoapError(h, 501, "ActionFailed");
-		ClearNameValueList(&data);
-		return ;
-	}
-	ClearNameValueList(&data);
 }
 
 static void
 GetPinholePackets(struct upnphttp * h, const char * action)
 {
-	if(CheckStatus(h)==0)
-		return;
-	int r, n;
-
 	static const char resp[] =
 		"<u:%sResponse "
 		"xmlns:u=\"%s\">"
-		"<PinholePackets>%d</PinholePackets>"
+		"<PinholePackets>%u</PinholePackets>"
 		"</u:%sResponse>";
-
 	char body[512];
 	int bodylen;
 	struct NameValueParserData data;
-	const char * uid;
-	char iaddr[40], proto[6], lt[12];
+	const char * uid_str;
+	int n;
+	char iaddr[INET6_ADDRSTRLEN];
 	unsigned short iport;
-	int pinholePackets = 0;
+	unsigned int packets = 0;
+	int uid;
+	int proto;
+	unsigned int leasetime;
+
+	if(CheckStatus(h)==0)
+		return;
 
 	ParseNameValue(h->req_buf + h->req_contentoff, h->req_contentlen, &data);
-	uid = GetValueFromNameValueList(&data, "UniqueID");
+	uid_str = GetValueFromNameValueList(&data, "UniqueID");
+	uid = uid_str ? atoi(uid_str) : -1;
+	ClearNameValueList(&data);
 
-	if(!uid)
+	if(uid < 0 || uid > 65535)
 	{
-		ClearNameValueList(&data);
 		SoapError(h, 402, "Invalid Args");
 		return;
 	}
 
 	/* Check that client is not getting infos of a pinhole
 	 * it doesn't have access to, because of its public access */
-	r = upnp_get_pinhole_info(0, 0, iaddr, &iport, proto, uid, lt);
-	if (r > 0)
+	n = upnp_get_pinhole_info(uid, NULL, 0, NULL,
+	                          iaddr, sizeof(iaddr), &iport,
+	                          &proto, &leasetime, &packets);
+	if (n >= 0)
 	{
-		if(PinholeVerification(h, iaddr, &iport)==0)
-		{
-			ClearNameValueList(&data);
+		if(PinholeVerification(h, iaddr, iport)<=0)
 			return ;
-		}
-	}
-
-	n = upnp_get_pinhole_packets(uid, &pinholePackets);
-	if(n > 0)
-	{
-		bodylen = snprintf(body, sizeof(body), resp,
-				action, "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1",
-				pinholePackets, action);
-		BuildSendAndCloseSoapResp(h, body, bodylen);
 	}
+#if 0
 	else if(r == -4 || r == -1)
 	{
 		SoapError(h, 704, "NoSuchEntry");
 	}
-	else
-	{
-		SoapError(h, 501, "ActionFailed");
-		ClearNameValueList(&data);
-		return ;
-	}
-	ClearNameValueList(&data);
+#endif
+
+	bodylen = snprintf(body, sizeof(body), resp,
+			action, "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1",
+			packets, action);
+	BuildSendAndCloseSoapResp(h, body, bodylen);
 }
 #endif
 
@@ -1776,13 +1795,13 @@ soapMethods[] =
 #endif
 #ifdef ENABLE_6FC_SERVICE
 	/* WANIPv6FirewallControl */
-	{ "GetFirewallStatus", GetFirewallStatus},
-	{ "AddPinhole", AddPinhole},
-	{ "UpdatePinhole", UpdatePinhole},
-	{ "GetOutboundPinholeTimeout", GetOutboundPinholeTimeout},
-	{ "DeletePinhole", DeletePinhole},
-	{ "CheckPinholeWorking", CheckPinholeWorking},
-	{ "GetPinholePackets", GetPinholePackets},
+	{ "GetFirewallStatus", GetFirewallStatus},	/* Required */
+	{ "AddPinhole", AddPinhole},				/* Required */
+	{ "UpdatePinhole", UpdatePinhole},			/* Required */
+	{ "GetOutboundPinholeTimeout", GetOutboundPinholeTimeout},	/* Optional */
+	{ "DeletePinhole", DeletePinhole},			/* Required */
+	{ "CheckPinholeWorking", CheckPinholeWorking},	/* Optional */
+	{ "GetPinholePackets", GetPinholePackets},	/* Required */
 #endif
 	{ 0, 0 }
 };
